# Define pkgdocdir for releases that don't define it already
%{!?_pkgdocdir: %global _pkgdocdir %{_docdir}/%{name}-%{version}}

# determine whether to install systemd or sysvinit scripts
%if 0%{?fedora} || 0%{?rhel} >= 7

%global _use_sysvinit 0
%global _use_systemd 1

%else

%global _use_sysvinit 1
%global _use_systemd 0

%endif

%global proton_minimum_version 0.33.0
%global libwebsockets_minimum_version 3.2.3
%global libnghttp2_minimum_version 1.33.0

%global embedded_proton_version qpid-proton-0.35.0

%undefine __brp_mangle_shebangs

Name:          qpid-dispatch
Version:       1.16.1
Release:       1%{?dist}
Summary:       Dispatch router for Qpid
License:       ASL 2.0
URL:           http://qpid.apache.org/
Source0:       http://www.apache.org/dist/qpid/dispatch/%{version}/qpid-dispatch-%{version}.tar.gz

Source2:       licenses.xml
Source3:       qpid-dispatch-console-%{version}.tar.gz
Source4:       config.json

# proton
Source5:       %{embedded_proton_version}.tar.gz
Source6:       protonbuild.patch

%global _pkglicensedir %{_licensedir}/%{name}-%{version}
%{!?_licensedir:%global license %doc}
%{!?_licensedir:%global _pkglicensedir %{_pkgdocdir}}

%if 0%{?rhel} && 0%{?rhel} >= 7
ExcludeArch: i686
%endif

%if 0%{?rhel} && 0%{?rhel} <= 6
Source1:       docs-%{version}-1.tar.gz
%endif

Patch3:        dispatch.patch
Patch4:        console-listener.patch

# proton deps (not building bindings so don't need python, ruby)
BuildRequires:  gcc
BuildRequires:  gcc-c++
BuildRequires:  cmake
BuildRequires:  pkgconfig
BuildRequires:  libuuid-devel
BuildRequires:  openssl-devel
BuildRequires:  cyrus-sasl-devel

# router deps
BuildRequires: qpid-proton-c-devel >= %{proton_minimum_version}
BuildRequires: python3-qpid-proton >= %{proton_minimum_version}
BuildRequires: python3-devel
BuildRequires: cmake
BuildRequires: openssl-devel
BuildRequires: libwebsockets-devel = %{libwebsockets_minimum_version}
BuildRequires: libnghttp2-devel >= %{libnghttp2_minimum_version}

# Missing dependency on RHEL 6: asciidoc >= 8.6.8
# asciidoc-8.4.5-4.1.el6 does not work for man pages
%if 0%{?fedora} || (0%{?rhel} && 0%{?rhel} >= 7)
BuildRequires: asciidoc >= 8.6.8
BuildRequires: systemd
%endif


%description
A lightweight message router, written in C and built on Qpid Proton, that provides flexible and scalable interconnect between AMQP endpoints or between endpoints and brokers.


%package router
Summary:  The Qpid Dispatch Router executable
Obsoletes: libqpid-dispatch
Obsoletes: libqpid-dispatch-devel
Requires:  python3

# proton
#Requires:  qpid-proton-c%{?_isa} >= %{proton_minimum_version}
Requires:  python3-qpid-proton >= %{proton_minimum_version}

%if %{_use_systemd}
Requires(post): systemd
Requires(preun): systemd
Requires(postun): systemd
%endif
Requires: libwebsockets = %{libwebsockets_minimum_version}
Requires: libnghttp2 >= %{libnghttp2_minimum_version}

%description router
%{summary}.


%files router
%license %{_pkglicensedir}/LICENSE
%license %{_pkglicensedir}/licenses.xml
%{_sbindir}/qdrouterd
%config(noreplace) %{_sysconfdir}/qpid-dispatch/qdrouterd.conf
%config(noreplace) %{_sysconfdir}/sasl2/qdrouterd.conf
%{_exec_prefix}/lib/qpid-dispatch
%{python3_sitelib}/qpid_dispatch_site.py*
%{python3_sitelib}/qpid_dispatch
%{python3_sitelib}/qpid_dispatch-*.egg-info
%{python3_sitelib}/__pycache__/*

%if %{_use_systemd}

%{_unitdir}/qdrouterd.service

%else

%{_initrddir}/qdrouterd
%attr(755,qdrouterd,qdrouterd) %dir %{_localstatedir}/run/qpid-dispatch

%endif

%{_mandir}/man5/qdrouterd.conf.5*
%{_mandir}/man8/qdrouterd.8*

%pre router
getent group qdrouterd >/dev/null || groupadd -r qdrouterd
getent passwd qdrouterd >/dev/null || \
  useradd -r -M -g qdrouterd -d %{_localstatedir}/lib/qdrouterd -s /sbin/nologin \
    -c "Owner of Qdrouterd Daemons" qdrouterd
exit 0


%if %{_use_systemd}

%post router
/sbin/ldconfig
%systemd_post qdrouterd.service

%preun router
%systemd_preun qdrouterd.service

%postun router
/sbin/ldconfig
%systemd_postun_with_restart qdrouterd.service

%endif

%if %{_use_sysvinit}

%post router
/sbin/ldconfig
/sbin/chkconfig --add qdrouterd

%preun router
if [ $1 -eq 0 ]; then
    /sbin/service qdrouterd stop >/dev/null 2>&1
    /sbin/chkconfig --del qdrouterd
fi

%postun router
/sbin/ldconfig
if [ "$1" -ge "1" ]; then
    /sbin/service qdrouterd condrestart >/dev/null 2>&1
fi

%endif


%package docs
Summary:   Documentation for the Qpid Dispatch router
BuildArch: noarch
Obsoletes:  qpid-dispatch-router-docs

%description docs
%{summary}.


%files docs
%doc %{_pkgdocdir}
%license %{_pkglicensedir}/LICENSE
%license %{_pkglicensedir}/licenses.xml


%package console
Summary: Web console for Qpid Dispatch Router
BuildArch: noarch
Requires: qpid-dispatch-router

%description console
%{summary}.

%files console
%{_datarootdir}/qpid-dispatch/console


%package tools
Summary: Tools for the Qpid Dispatch router
BuildArch: noarch

# proton
Requires: python3-qpid-proton >= %{proton_minimum_version}


%description tools
%{summary}.


%files tools
%{_bindir}/qdstat
%{_bindir}/qdmanage

%{_mandir}/man8/qdstat.8*
%{_mandir}/man8/qdmanage.8*


%prep
%setup -q -b 5

%patch3 -p1
%patch4 -p1

{
    mkdir pre_built
    cd pre_built
    tar xvzpf %{SOURCE3} -C .
    cp %{SOURCE4} console/
    %if 0%{?rhel} && 0%{?rhel} <= 6
      tar xvzpf %{SOURCE1} -C .
    %endif
    cd ..
}

# downstream only - patch the proton make
pushd %{_builddir}/%{embedded_proton_version}
patch -p1 < %{SOURCE6}
popd

%build
(
    cd %{_builddir}/%{embedded_proton_version}
    mkdir build
    cd build

    # Need ENABLE_FUZZ_TESTING=NO to avoid a link failure
    %cmake  \
          -DBUILD_BINDINGS=OFF \
          -DBUILD_STATIC_LIBS=ON \
          -DENABLE_FUZZ_TESTING=NO \
          -DCMAKE_INSTALL_PREFIX:PATH=%{_builddir}/%{embedded_proton_version}/install \
          -DLIB_INSTALL_DIR:PATH=%{_builddir}/%{embedded_proton_version}/install/usr/lib64 \
          -DINCLUDE_INSTALL_DIR:PATH=%{_builddir}/%{embedded_proton_version}/install/usr/include \
          ..

    make -j4 install
)


export DOCS=ON
%if 0%{?rhel} && 0%{?rhel} <= 6
export DOCS=OFF
%endif
%cmake -DDOC_INSTALL_DIR=%{?_pkgdocdir} \
       -DCMAKE_BUILD_TYPE=RelWithDebInfo \
       -DUSE_SETUP_PY=0 \
       -DQD_DOC_INSTALL_DIR=%{_pkgdocdir} \
       "-DBUILD_DOCS=$DOCS" \
       -DCMAKE_SKIP_RPATH:BOOL=OFF \
       -DUSE_LIBWEBSOCKETS=ON \
       -DCONSOLE_INSTALL=OFF \
       -DProton_USE_STATIC_LIBS=ON \
       -DProton_DIR=%{_builddir}/%{embedded_proton_version}/install/usr/lib64/cmake/Proton \
       "-DCMAKE_C_FLAGS=$CMAKE_CXX_FLAGS $CFLAGS -Wno-error=deprecated-declarations" \
       .

# to fix multilib install, force (any) known date to generated file (and patched .py files)
#touch -r %{SOURCE4} %{_builddir}/qpid-dispatch-%{version}/tests/system_tests_edge_router.py
#touch -r %{SOURCE4} %{_builddir}/qpid-dispatch-%{version}/tests/system_tests_one_router.py
#touch -r %{SOURCE4} %{_builddir}/qpid-dispatch-%{version}/tests/system_tests_ssl.py

make doc


%install
%make_install

%if %{_use_systemd}

install -dm 755 %{buildroot}%{_unitdir}
install -pm 644 %{_builddir}/qpid-dispatch-%{version}/etc/fedora/qdrouterd.service \
                %{buildroot}%{_unitdir}

%else

install -dm 755 %{buildroot}%{_initrddir}
install -pm 755 %{_builddir}/qpid-dispatch-%{version}/etc/fedora/qdrouterd \
                %{buildroot}%{_initrddir}

%endif

%if 0%{?rhel} && 0%{?rhel} <= 6
install -dm 755 %{buildroot}%{_mandir}/man5
install -dm 755 %{buildroot}%{_mandir}/man8
install -pm 644 %{_builddir}/qpid-dispatch-%{version}/pre_built/man/man5/* %{buildroot}%{_mandir}/man5/
install -pm 644 %{_builddir}/qpid-dispatch-%{version}/pre_built/man/man8/* %{buildroot}%{_mandir}/man8/
cp -a           %{_builddir}/qpid-dispatch-%{version}/pre_built/doc/qpid-dispatch-%{version}/*  \
                %{buildroot}%{_pkgdocdir}/
%endif

install -dm 755 %{buildroot}/var/run/qpid-dispatch

%if 0%{?rhel} && 0%{?rhel} <= 6
install -pm 644 %{SOURCE2} %{buildroot}%{_pkgdocdir}
%else
install -dm 755 %{buildroot}%{_pkglicensedir}
install -pm 644 %{SOURCE2} %{buildroot}%{_pkglicensedir}
install -pm 644 %{buildroot}%{_pkgdocdir}/LICENSE %{buildroot}%{_pkglicensedir}
rm -f %{buildroot}%{_pkgdocdir}/LICENSE
%endif

install -dm 755 %{buildroot}/%{_datarootdir}/qpid-dispatch/console
cp -a %{_builddir}/qpid-dispatch-%{version}/pre_built/console/* %{buildroot}/%{_datarootdir}/qpid-dispatch/console/

rm -f  %{buildroot}/%{_includedir}/qpid/dispatch.h
rm -fr %{buildroot}/%{_includedir}/qpid/dispatch

%post -p /sbin/ldconfig

%postun -p /sbin/ldconfig


%changelog
* Tue Jul 6 2021 Mike Cressman <mcressma@redhat.com> - 1.16.1-1
- DISPATCH-2142 - improvements for tcp buffering
- DISPATCH-2097 - don't display internal-only connections on the console
- use proton 0.35.0 internally

* Tue Jun 8 2021 Mike Cressman <mcressma@redhat.com> - 1.16.0-4
- DISPATCH-1878 - account for new dispositions used for flow control

* Fri May 21 2021 Mike Cressman <mcressma@redhat.com> - 1.16.0-3
- Rebase to upstream 1.16.0
- Compile proton libs internally (v0.34.0)

* Thu Mar 25 2021 Mike Cressman <mcressma@redhat.com> - 1.15.0-4
- Updated console dependencies

* Wed Mar 10 2021 Mike Cressman <mcressma@redhat.com> - 1.15.0-3
- DISPATCH-1988 - fix router crash when sending an empty transfer frame followed by an abort

* Wed Feb 24 2021 Mike Cressman <mcressma@redhat.com> - 1.15.0-2
- DISPATCH-1960,1961,1947 - add Q2 flow control for protocol adaptors

* Fri Feb 12 2021 Mike Cressman <mcressma@redhat.com> - 1.15.0-1
- Rebase to upstream 1.15.0

* Mon Jan 11 2021 Mike Cressman <mcressma@redhat.com> - 1.14.0-1
- Rebase to upstream 1.14.0

* Tue Sep 8 2020 Mike Cressman <mcressma@redhat.com> - 1.13.0-3
- Fix DISPATCH-1762 - Connector ssl config errors must prohibit connections
- Fix DISPATCH-1765 - Add streaming links to offered capabilities

* Tue Aug 18 2020 Mike Cressman <mcressma@redhat.com> - 1.13.0-1
- Rebase to upstream 1.13.0

* Thu Jun 18 2020 Mike Cressman <mcressma@redhat.com> - 1.12.0-5
- Fix DISPATCH-1637 - Add hashtable to store names of link routes, auto-links and address configs
- Fix DISPATCH-1642 - Change parse tree lookup to fail on duplicate entry
- Fix DISPATCH-1682 - Augment parse tree with hash table lookup

* Mon Jun 1 2020 Mike Cressman <mcressma@redhat.com> - 1.12.0-4
- Fix DISPATCH-1664 - Fix format specifier for policy count logs

* Wed May 20 2020 Mike Cressman <mcressma@redhat.com> - 1.12.0-2
- Fix DISPATCH-1614 - Edge router crash when interior closes edge uplink connection
- Fix DISPATCH-1641 - Add error condition to rejected oversized dispositions
- Misc doc and test updates

* Tue May 5 2020 Mike Cressman <mcressma@redhat.com> - 1.12.0-1
- Rebase to upstream 1.12.0

* Mon Feb 10 2020 Mike Cressman <mcressma@redhat.com> - 1.10.0-2
- Revert to libwebsockets 2.4.2 for now
- Fix ENTMQIC-2410,DISPATCH-1557 - delay free of pn_link_t until after event loop is complete

* Mon Jan 13 2020 Mike Cressman <mcressma@redhat.com> - 1.10.0-1
- Rebase to upstream 1.10.0

* Fri Dec 6 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-6
- Fix DISPATCH-1512 - fix bug with DISPATCH-1500

* Wed Dec 4 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-5
- Fix DISPATCH-1500 - improve efficiency of parsing and processing for large MAU

* Wed Nov 27 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-4
- Fix DISPATCH-1483 - link counters for dispositions are not incremented for outgoing links
- Fix DISPATCH-1491 - bottleneck adding or removing addresses in mobile address engine

* Tue Nov 19 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-3
- Fix DISPATCH-1460 - router control messages for local subscribers hang in Q2 holdoff
- Fix DISPATCH-1461 - correct the logging of long addresses on attach

* Fri Oct 4 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-2
- Fix DISPATCH-1423 - restore original multicast credit replenish behavior

* Tue Sep 24 2019 Mike Cressman <mcressma@redhat.com> - 1.9.0-1
- Rebased to upstream 1.9.0
- Fix multilib issue with generated python file

* Fri Sep 13 2019 Mike Cressman <mcressma@redhat.com> - 1.8.0-5
- Fixed ENTMQIC-2330 - Memory leak on link policy denial
- Fixed ENTMQIC-2318,2319 - Proton link reference not cleared on router link objects during session close

* Fri Jun 14 2019 Ganesh Murthy <gmurthy@redhat.com> - 1.8.0-2
- Rebased to upstream 1.8.0
- Added fixes DISPATCH-1361 and DISPATCH-1362
- Require libwebsockets 2.4.2 exact

* Mon May 13 2019 Mike Cressman <mcressma@redhat.com> - 1.7.0-2
- Fixed ENTMQIC-2200, ENTMQIC-2201, ENTMQIC-2202, ENTMQIC-2203

* Mon Apr 15 2019 Irina Boverman <iboverma@redhat.com> - 1.7.0-1
- Rebased to upstream 1.7.0

* Wed Mar 27 2019 Ted Ross <tross@redhat.com> - 1.6.0-1
- Rebased to upstream 1.6.0
- Built against qpid-proton 0.27.0

* Thu Jan 10 2019 Ted Ross <tross@redhat.com> - 1.5.0-4
- Fixed ENTMQIC-2092, ENTMQIC-2093, ENTMQIC-2094, and ENTMQIC-2095

* Thu Dec 20 2018 Irina Boverman <iboverma@redhat.com> - 1.5.0-3
- Rebuilt againts qpid-proton 0.26.0-3 and updated licenses.xml file

* Wed Dec 19 2018 Ted Ross <tross@redhat.com> - 1.5.0-2
- Rebased to official upstream 1.5.0

* Fri Nov 2 2018 Ted Ross <tross@redhat.com> - 1.5.0-1
- Rebased to 1.5.0-SNAPSHOT
- Initial build on RHEL 8 agaist proton 0.26.0
